@@ -251,9 +251,11 @@ void Clip::AttachToObject(std::string object_id)
 		// Check for valid tracked object
 		if (trackedObject){
 			SetAttachedObject(trackedObject);
+            parentClipObject = NULL;
 		}
 		else if (clipObject) {
 			SetAttachedClip(clipObject);
+            parentTrackedObject = nullptr;
 		}
 	}
 }
@@ -414,49 +416,48 @@ std::shared_ptr<Frame> Clip::GetFrame(std::shared_ptr<openshot::Frame> backgroun
 
 		// Check cache
 		frame = final_cache.GetFrame(clip_frame_number);
-		if (frame) {
-			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod(
-					"Clip::GetFrame (Cached frame found)",
-					"requested_frame", clip_frame_number);
+		if (!frame) {
+            // Generate clip frame
+            frame = GetOrCreateFrame(clip_frame_number);
 
-			// Return cached frame
-			return frame;
-		}
+            // Get frame size and frame #
+            int64_t timeline_frame_number = clip_frame_number;
+            QSize timeline_size(frame->GetWidth(), frame->GetHeight());
+            if (background_frame) {
+                // If a background frame is provided, use it instead
+                timeline_frame_number = background_frame->number;
+                timeline_size.setWidth(background_frame->GetWidth());
+                timeline_size.setHeight(background_frame->GetHeight());
+            }
 
-		// Generate clip frame
-		frame = GetOrCreateFrame(clip_frame_number);
+            // Get time mapped frame object (used to increase speed, change direction, etc...)
+            apply_timemapping(frame);
 
-		if (!background_frame) {
-			// Create missing background_frame w/ transparent color (if needed)
-			background_frame = std::make_shared<Frame>(clip_frame_number, frame->GetWidth(), frame->GetHeight(),
-													   "#00000000",  frame->GetAudioSamplesCount(),
-													   frame->GetAudioChannelsCount());
-		}
+            // Apply waveform image (if any)
+            apply_waveform(frame, timeline_size);
 
-		// Get time mapped frame object (used to increase speed, change direction, etc...)
-		apply_timemapping(frame);
+            // Apply effects BEFORE applying keyframes (if any local or global effects are used)
+            apply_effects(frame, timeline_frame_number, options, true);
 
-		// Apply waveform image (if any)
-		apply_waveform(frame, background_frame);
+            // Apply keyframe / transforms to current clip image
+            apply_keyframes(frame, timeline_size);
 
-		// Apply local effects to the frame (if any)
-		apply_effects(frame);
+            // Apply effects AFTER applying keyframes (if any local or global effects are used)
+            apply_effects(frame, timeline_frame_number, options, false);
 
-		// Apply global timeline effects (i.e. transitions & masks... if any)
-		if (timeline != NULL && options != NULL) {
-			if (options->is_top_clip) {
-				// Apply global timeline effects (only to top clip... if overlapping, pass in timeline frame number)
-				Timeline* timeline_instance = static_cast<Timeline*>(timeline);
-				frame = timeline_instance->apply_effects(frame, background_frame->number, Layer());
-			}
-		}
+            // Add final frame to cache (before flattening into background_frame)
+            final_cache.Add(frame);
+        }
 
-		// Apply keyframe / transforms
-		apply_keyframes(frame, background_frame);
+        if (!background_frame) {
+            // Create missing background_frame w/ transparent color (if needed)
+            background_frame = std::make_shared<Frame>(frame->number, frame->GetWidth(), frame->GetHeight(),
+                                                       "#00000000",  frame->GetAudioSamplesCount(),
+                                                       frame->GetAudioChannelsCount());
+        }
 
-		// Add final frame to cache
-		final_cache.Add(frame);
+		// Apply background canvas (i.e. flatten this image onto previous layer image)
+		apply_background(frame, background_frame);
 
 		// Return processed 'frame'
 		return frame;
@@ -478,6 +479,24 @@ openshot::EffectBase* Clip::GetEffect(const std::string& id)
 	return nullptr;
 }
 
+// Return the associated ParentClip (if any)
+openshot::Clip* Clip::GetParentClip() {
+    if (!parentObjectId.empty() && (!parentClipObject && !parentTrackedObject)) {
+        // Attach parent clip OR object to this clip
+        AttachToObject(parentObjectId);
+    }
+    return parentClipObject;
+}
+
+// Return the associated Parent Tracked Object (if any)
+std::shared_ptr<openshot::TrackedObjectBase> Clip::GetParentTrackedObject() {
+    if (!parentObjectId.empty() && (!parentClipObject && !parentTrackedObject)) {
+        // Attach parent clip OR object to this clip
+        AttachToObject(parentObjectId);
+    }
+    return parentTrackedObject;
+}
+
 // Get file extension
 std::string Clip::get_file_extension(std::string path)
 {
@@ -485,33 +504,6 @@ std::string Clip::get_file_extension(std::string path)
 	return path.substr(path.find_last_of(".") + 1);
 }
 
-// Reverse an audio buffer
-void Clip::reverse_buffer(juce::AudioBuffer<float>* buffer)
-{
-	int number_of_samples = buffer->getNumSamples();
-	int channels = buffer->getNumChannels();
-
-	// Reverse array (create new buffer to hold the reversed version)
-	auto *reversed = new juce::AudioBuffer<float>(channels, number_of_samples);
-	reversed->clear();
-
-	for (int channel = 0; channel < channels; channel++)
-	{
-		int n=0;
-		for (int s = number_of_samples - 1; s >= 0; s--, n++)
-			reversed->getWritePointer(channel)[n] = buffer->getWritePointer(channel)[s];
-	}
-
-	// Copy the samples back to the original array
-	buffer->clear();
-	// Loop through channels, and get audio samples
-	for (int channel = 0; channel < channels; channel++)
-		// Get the audio samples for this channel
-		buffer->addFrom(channel, 0, reversed->getReadPointer(channel), number_of_samples, 1.0f);
-
-	delete reversed;
-}
-
 // Adjust the audio and image of a time mapped frame
 void Clip::apply_timemapping(std::shared_ptr<Frame> frame)
 {
@@ -788,38 +780,8 @@ std::string Clip::PropertiesJSON(int64_t requested_frame) const {
 	root["waveform"]["choices"].append(add_property_choice_json("Yes", true, waveform));
 	root["waveform"]["choices"].append(add_property_choice_json("No", false, waveform));
 
-	// Add the parentTrackedObject's properties
-	if (parentTrackedObject && parentClipObject)
-	{
-		// Convert Clip's frame position to Timeline's frame position
-		long clip_start_position = round(Position() * info.fps.ToDouble()) + 1;
-		long clip_start_frame = (Start() * info.fps.ToDouble()) + 1;
-		double timeline_frame_number = requested_frame + clip_start_position - clip_start_frame;
-
-		// Get attached object's parent clip properties
-		std::map< std::string, float > trackedObjectParentClipProperties = parentTrackedObject->GetParentClipProperties(timeline_frame_number);
-		double parentObject_frame_number = trackedObjectParentClipProperties["frame_number"];
-		// Get attached object properties
-		std::map< std::string, float > trackedObjectProperties = parentTrackedObject->GetBoxValues(parentObject_frame_number);
-
-		// Correct the parent Tracked Object properties by the clip's reference system
-		float parentObject_location_x = trackedObjectProperties["cx"] - 0.5 + trackedObjectParentClipProperties["cx"];
-		float parentObject_location_y = trackedObjectProperties["cy"] - 0.5 + trackedObjectParentClipProperties["cy"];
-		float parentObject_scale_x = trackedObjectProperties["w"]*trackedObjectProperties["sx"];
-		float parentObject_scale_y = trackedObjectProperties["h"]*trackedObjectProperties["sy"];
-		float parentObject_rotation = trackedObjectProperties["r"] + trackedObjectParentClipProperties["r"];
-
-		// Add the parent Tracked Object properties to JSON
-		root["location_x"] = add_property_json("Location X", parentObject_location_x, "float", "", &location_x, -1.0, 1.0, false, requested_frame);
-		root["location_y"] = add_property_json("Location Y", parentObject_location_y, "float", "", &location_y, -1.0, 1.0, false, requested_frame);
-		root["scale_x"] = add_property_json("Scale X", parentObject_scale_x, "float", "", &scale_x, 0.0, 1.0, false, requested_frame);
-		root["scale_y"] = add_property_json("Scale Y", parentObject_scale_y, "float", "", &scale_y, 0.0, 1.0, false, requested_frame);
-		root["rotation"] = add_property_json("Rotation", parentObject_rotation, "float", "", &rotation, -360, 360, false, requested_frame);
-		root["shear_x"] = add_property_json("Shear X", shear_x.GetValue(requested_frame), "float", "", &shear_x, -1.0, 1.0, false, requested_frame);
-		root["shear_y"] = add_property_json("Shear Y", shear_y.GetValue(requested_frame), "float", "", &shear_y, -1.0, 1.0, false, requested_frame);
-	}
 	// Add the parentClipObject's properties
-	else if (parentClipObject)
+	if (parentClipObject)
 	{
 		// Convert Clip's frame position to Timeline's frame position
 		long clip_start_position = round(Position() * info.fps.ToDouble()) + 1;
@@ -1043,6 +1005,11 @@ void Clip::SetJsonValue(const Json::Value root) {
 
 		// loop through effects
 		for (const auto existing_effect : root["effects"]) {
+			// Skip NULL nodes
+			if (existing_effect.isNull()) {
+				continue;
+			}
+
 			// Create Effect
 			EffectBase *e = NULL;
 			if (!existing_effect["type"].isNull()) {
@@ -1202,46 +1169,67 @@ void Clip::RemoveEffect(EffectBase* effect)
 	final_cache.Clear();
 }
 
+// Apply background image to the current clip image (i.e. flatten this image onto previous layer)
+void Clip::apply_background(std::shared_ptr<openshot::Frame> frame, std::shared_ptr<openshot::Frame> background_frame) {
+	// Add background canvas
+	std::shared_ptr<QImage> background_canvas = background_frame->GetImage();
+	QPainter painter(background_canvas.get());
+	painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform | QPainter::TextAntialiasing, true);
+
+	// Composite a new layer onto the image
+	painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
+	painter.drawImage(0, 0, *frame->GetImage());
+	painter.end();
+
+	// Add new QImage to frame
+	frame->AddImage(background_canvas);
+}
+
 // Apply effects to the source frame (if any)
-void Clip::apply_effects(std::shared_ptr<Frame> frame)
+void Clip::apply_effects(std::shared_ptr<Frame> frame, int64_t timeline_frame_number, TimelineInfoStruct* options, bool before_keyframes)
 {
-	// Find Effects at this position and layer
 	for (auto effect : effects)
 	{
 		// Apply the effect to this frame
-		frame = effect->GetFrame(frame, frame->number);
+		if (effect->info.apply_before_clip && before_keyframes) {
+			effect->GetFrame(frame, frame->number);
+		} else if (!effect->info.apply_before_clip && !before_keyframes) {
+			effect->GetFrame(frame, frame->number);
+		}
+	}
 
-	} // end effect loop
+	if (timeline != NULL && options != NULL) {
+		// Apply global timeline effects (i.e. transitions & masks... if any)
+		Timeline* timeline_instance = static_cast<Timeline*>(timeline);
+		options->is_before_clip_keyframes = before_keyframes;
+		timeline_instance->apply_effects(frame, timeline_frame_number, Layer(), options);
+	}
 }
 
 // Compare 2 floating point numbers for equality
-bool Clip::isEqual(double a, double b)
+bool Clip::isNear(double a, double b)
 {
 	return fabs(a - b) < 0.000001;
 }
 
 // Apply keyframes to the source frame (if any)
-void Clip::apply_keyframes(std::shared_ptr<Frame> frame, std::shared_ptr<Frame> background_frame) {
+void Clip::apply_keyframes(std::shared_ptr<Frame> frame, QSize timeline_size) {
 	// Skip out if video was disabled or only an audio frame (no visualisation in use)
 	if (!frame->has_image_data) {
 		// Skip the rest of the image processing for performance reasons
 		return;
 	}
 
-	// Get image from clip
+	// Get image from clip, and create transparent background image
 	std::shared_ptr<QImage> source_image = frame->GetImage();
-	std::shared_ptr<QImage> background_canvas = background_frame->GetImage();
+	std::shared_ptr<QImage> background_canvas = std::make_shared<QImage>(timeline_size.width(),
+                                                                         timeline_size.height(),
+																		 QImage::Format_RGBA8888_Premultiplied);
+	background_canvas->fill(QColor(Qt::transparent));
 
 	// Get transform from clip's keyframes
 	QTransform transform = get_transform(frame, background_canvas->width(), background_canvas->height());
 
-	// Debug output
-	ZmqLogger::Instance()->AppendDebugMethod(
-		"Clip::ApplyKeyframes (Transform: Composite Image Layer: Prepare)",
-		"frame->number", frame->number,
-		"background_canvas->width()", background_canvas->width(),
-		"background_canvas->height()", background_canvas->height());
-
 	// Load timeline's new frame image into a QPainter
 	QPainter painter(background_canvas.get());
 	painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform | QPainter::TextAntialiasing, true);
@@ -1289,7 +1277,7 @@ void Clip::apply_keyframes(std::shared_ptr<Frame> frame, std::shared_ptr<Frame>
 }
 
 // Apply apply_waveform image to the source frame (if any)
-void Clip::apply_waveform(std::shared_ptr<Frame> frame, std::shared_ptr<Frame> background_frame) {
+void Clip::apply_waveform(std::shared_ptr<Frame> frame, QSize timeline_size) {
 
 	if (!Waveform()) {
 		// Exit if no waveform is needed
@@ -1298,15 +1286,13 @@ void Clip::apply_waveform(std::shared_ptr<Frame> frame, std::shared_ptr<Frame> b
 
 	// Get image from clip
 	std::shared_ptr<QImage> source_image = frame->GetImage();
-	std::shared_ptr<QImage> background_canvas = background_frame->GetImage();
 
 	// Debug output
-	ZmqLogger::Instance()->AppendDebugMethod(
-			"Clip::apply_waveform (Generate Waveform Image)",
+	ZmqLogger::Instance()->AppendDebugMethod("Clip::apply_waveform (Generate Waveform Image)",
 			"frame->number", frame->number,
 			"Waveform()", Waveform(),
-			"background_canvas->width()", background_canvas->width(),
-			"background_canvas->height()", background_canvas->height());
+			"width", timeline_size.width(),
+			"height", timeline_size.height());
 
 	// Get the color of the waveform
 	int red = wave_color.red.GetInt(frame->number);
@@ -1315,11 +1301,32 @@ void Clip::apply_waveform(std::shared_ptr<Frame> frame, std::shared_ptr<Frame> b
 	int alpha = wave_color.alpha.GetInt(frame->number);
 
 	// Generate Waveform Dynamically (the size of the timeline)
-	source_image = frame->GetWaveform(background_canvas->width(), background_canvas->height(), red, green, blue, alpha);
+	source_image = frame->GetWaveform(timeline_size.width(), timeline_size.height(), red, green, blue, alpha);
 	frame->AddImage(source_image);
 }
 
-// Apply keyframes to the source frame (if any)
+// Scale a source size to a target size (given a specific scale-type)
+QSize Clip::scale_size(QSize source_size, ScaleType source_scale, int target_width, int target_height) {
+    switch (source_scale)
+    {
+        case (SCALE_FIT): {
+            source_size.scale(target_width, target_height, Qt::KeepAspectRatio);
+            break;
+        }
+        case (SCALE_STRETCH): {
+            source_size.scale(target_width, target_height, Qt::IgnoreAspectRatio);
+            break;
+        }
+        case (SCALE_CROP): {
+            source_size.scale(target_width, target_height, Qt::KeepAspectRatioByExpanding);;
+            break;
+        }
+    }
+
+    return source_size;
+}
+
+// Get QTransform from keyframes
 QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int height)
 {
 	// Get image from clip
@@ -1345,68 +1352,13 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 		}
 
 		// Debug output
-		ZmqLogger::Instance()->AppendDebugMethod(
-			"Clip::get_transform (Set Alpha & Opacity)",
+		ZmqLogger::Instance()->AppendDebugMethod("Clip::get_transform (Set Alpha & Opacity)",
 			"alpha_value", alpha_value,
 			"frame->number", frame->number);
 	}
 
 	/* RESIZE SOURCE IMAGE - based on scale type */
-	QSize source_size = source_image->size();
-
-	// Apply stretch scale to correctly fit the bounding-box
-	if (parentTrackedObject){
-		scale = SCALE_STRETCH;
-	}
-
-	switch (scale)
-	{
-		case (SCALE_FIT): {
-			source_size.scale(width, height, Qt::KeepAspectRatio);
-
-			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod(
-				"Clip::get_transform (Scale: SCALE_FIT)",
-				"frame->number", frame->number,
-				"source_width", source_size.width(),
-				"source_height", source_size.height());
-			break;
-		}
-		case (SCALE_STRETCH): {
-			source_size.scale(width, height, Qt::IgnoreAspectRatio);
-
-			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod(
-				"Clip::get_transform (Scale: SCALE_STRETCH)",
-				"frame->number", frame->number,
-				"source_width", source_size.width(),
-				"source_height", source_size.height());
-			break;
-		}
-		case (SCALE_CROP): {
-			source_size.scale(width, height, Qt::KeepAspectRatioByExpanding);
-
-			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod(
-				"Clip::get_transform (Scale: SCALE_CROP)",
-				"frame->number", frame->number,
-				"source_width", source_size.width(),
-				"source_height", source_size.height());
-			break;
-		}
-		case (SCALE_NONE): {
-			// Image is already the original size (i.e. no scaling mode) relative
-			// to the preview window size (i.e. timeline / preview ratio). No further
-			// scaling is needed here.
-			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod(
-				"Clip::get_transform (Scale: SCALE_NONE)",
-				"frame->number", frame->number,
-				"source_width", source_size.width(),
-				"source_height", source_size.height());
-			break;
-		}
-	}
+	QSize source_size = scale_size(source_image->size(), scale, width, height);
 
 	// Initialize parent object's properties (Clip or Tracked Object)
 	float parentObject_location_x = 0.0;
@@ -1418,63 +1370,53 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 	float parentObject_rotation = 0.0;
 
 	// Get the parentClipObject properties
-	if (parentClipObject){
-
-		// Convert Clip's frame position to Timeline's frame position
-		long clip_start_position = round(Position() * info.fps.ToDouble()) + 1;
-		long clip_start_frame = (Start() * info.fps.ToDouble()) + 1;
-		double timeline_frame_number = frame->number + clip_start_position - clip_start_frame;
+	if (GetParentClip()){
+        // Get the start trim position of the parent clip
+        long parent_start_offset = parentClipObject->Start() * info.fps.ToDouble();
+        long parent_frame_number = frame->number + parent_start_offset;
 
 		// Get parent object's properties (Clip)
-		parentObject_location_x = parentClipObject->location_x.GetValue(timeline_frame_number);
-		parentObject_location_y = parentClipObject->location_y.GetValue(timeline_frame_number);
-		parentObject_scale_x = parentClipObject->scale_x.GetValue(timeline_frame_number);
-		parentObject_scale_y = parentClipObject->scale_y.GetValue(timeline_frame_number);
-		parentObject_shear_x = parentClipObject->shear_x.GetValue(timeline_frame_number);
-		parentObject_shear_y = parentClipObject->shear_y.GetValue(timeline_frame_number);
-		parentObject_rotation = parentClipObject->rotation.GetValue(timeline_frame_number);
+		parentObject_location_x = parentClipObject->location_x.GetValue(parent_frame_number);
+		parentObject_location_y = parentClipObject->location_y.GetValue(parent_frame_number);
+		parentObject_scale_x = parentClipObject->scale_x.GetValue(parent_frame_number);
+		parentObject_scale_y = parentClipObject->scale_y.GetValue(parent_frame_number);
+		parentObject_shear_x = parentClipObject->shear_x.GetValue(parent_frame_number);
+		parentObject_shear_y = parentClipObject->shear_y.GetValue(parent_frame_number);
+		parentObject_rotation = parentClipObject->rotation.GetValue(parent_frame_number);
 	}
 
-	// Get the parentTrackedObject properties
-	if (parentTrackedObject){
-		// Convert Clip's frame position to Timeline's frame position
-		long clip_start_position = round(Position() * info.fps.ToDouble()) + 1;
-		long clip_start_frame = (Start() * info.fps.ToDouble()) + 1;
-		double timeline_frame_number = frame->number + clip_start_position - clip_start_frame;
+    // Get the parentTrackedObject properties
+    if (GetParentTrackedObject()){
+        // Get the attached object's parent clip's properties
+        Clip* parentClip = (Clip*) parentTrackedObject->ParentClip();
+        if (parentClip)
+        {
+            // Get the start trim position of the parent clip
+            long parent_start_offset = parentClip->Start() * info.fps.ToDouble();
+            long parent_frame_number = frame->number + parent_start_offset;
 
-		// Get parentTrackedObject's parent clip's properties
-		std::map<std::string, float> trackedObjectParentClipProperties =
-				parentTrackedObject->GetParentClipProperties(timeline_frame_number);
+            // Access the parentTrackedObject's properties
+            std::map<std::string, float> trackedObjectProperties = parentTrackedObject->GetBoxValues(parent_frame_number);
 
-		// Get the attached object's parent clip's properties
-		if (!trackedObjectParentClipProperties.empty())
-		{
-			// Get parent object's properties (Tracked Object)
-			float parentObject_frame_number = trackedObjectParentClipProperties["frame_number"];
+            // Get actual scaled parent size
+            QSize parent_size = scale_size(QSize(parentClip->info.width, parentClip->info.height),
+                                           parentClip->scale, width, height);
 
-			// Access the parentTrackedObject's properties
-			std::map<std::string, float> trackedObjectProperties = parentTrackedObject->GetBoxValues(parentObject_frame_number);
+            // Get actual scaled tracked object size
+            int trackedWidth = trackedObjectProperties["w"] * trackedObjectProperties["sx"] * parent_size.width() *
+                    parentClip->scale_x.GetValue(parent_frame_number);
+            int trackedHeight = trackedObjectProperties["h"] * trackedObjectProperties["sy"] * parent_size.height() *
+                    parentClip->scale_y.GetValue(parent_frame_number);
 
-			// Get the Tracked Object's properties and correct them by the clip's reference system
-			parentObject_location_x = trackedObjectProperties["cx"] - 0.5 + trackedObjectParentClipProperties["location_x"];
-			parentObject_location_y = trackedObjectProperties["cy"] - 0.5 + trackedObjectParentClipProperties["location_y"];
-			parentObject_scale_x = trackedObjectProperties["w"]*trackedObjectProperties["sx"];
-			parentObject_scale_y = trackedObjectProperties["h"]*trackedObjectProperties["sy"];
-			parentObject_rotation = trackedObjectProperties["r"] + trackedObjectParentClipProperties["rotation"];
-		}
-		else
-		{
-			// Access the parentTrackedObject's properties
-			std::map<std::string, float> trackedObjectProperties = parentTrackedObject->GetBoxValues(timeline_frame_number);
+            // Scale the clip source_size based on the actual tracked object size
+            source_size = scale_size(source_size, scale, trackedWidth, trackedHeight);
 
-			// Get the Tracked Object's properties and correct them by the clip's reference system
-			parentObject_location_x = trackedObjectProperties["cx"] - 0.5;
-			parentObject_location_y = trackedObjectProperties["cy"] - 0.5;
-			parentObject_scale_x = trackedObjectProperties["w"]*trackedObjectProperties["sx"];
-			parentObject_scale_y = trackedObjectProperties["h"]*trackedObjectProperties["sy"];
-			parentObject_rotation = trackedObjectProperties["r"];
-		}
-	}
+            // Update parentObject's properties based on the tracked object's properties and parent clip's scale
+            parentObject_location_x = parentClip->location_x.GetValue(parent_frame_number) + ((trackedObjectProperties["cx"] - 0.5) * parentClip->scale_x.GetValue(parent_frame_number));
+            parentObject_location_y = parentClip->location_y.GetValue(parent_frame_number) + ((trackedObjectProperties["cy"] - 0.5) * parentClip->scale_y.GetValue(parent_frame_number));
+            parentObject_rotation = trackedObjectProperties["r"] + parentClip->rotation.GetValue(parent_frame_number);
+        }
+    }
 
 	/* GRAVITY LOCATION - Initialize X & Y to the correct values (before applying location curves) */
 	float x = 0.0; // left
@@ -1540,8 +1482,8 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 
 	/* LOCATION, ROTATION, AND SCALE */
 	float r = rotation.GetValue(frame->number) + parentObject_rotation; // rotate in degrees
-	x += (width * (location_x.GetValue(frame->number) + parentObject_location_x )); // move in percentage of final width
-	y += (height * (location_y.GetValue(frame->number) + parentObject_location_y )); // move in percentage of final height
+	x += width * (location_x.GetValue(frame->number) + parentObject_location_x); // move in percentage of final width
+	y += height * (location_y.GetValue(frame->number) + parentObject_location_y); // move in percentage of final height
 	float shear_x_value = shear_x.GetValue(frame->number) + parentObject_shear_x;
 	float shear_y_value = shear_y.GetValue(frame->number) + parentObject_shear_y;
 	float origin_x_value = origin_x.GetValue(frame->number);
@@ -1555,11 +1497,11 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 		"r", r,
 		"sx", sx, "sy", sy);
 
-	if (!isEqual(x, 0) || !isEqual(y, 0)) {
+	if (!isNear(x, 0) || !isNear(y, 0)) {
 		// TRANSLATE/MOVE CLIP
 		transform.translate(x, y);
 	}
-	if (!isEqual(r, 0) || !isEqual(shear_x_value, 0) || !isEqual(shear_y_value, 0)) {
+	if (!isNear(r, 0) || !isNear(shear_x_value, 0) || !isNear(shear_y_value, 0)) {
 		// ROTATE CLIP (around origin_x, origin_y)
 		float origin_x_offset = (scaled_source_width * origin_x_value);
 		float origin_y_offset = (scaled_source_height * origin_y_value);
@@ -1571,7 +1513,7 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 	// SCALE CLIP (if needed)
 	float source_width_scale = (float(source_size.width()) / float(source_image->width())) * sx;
 	float source_height_scale = (float(source_size.height()) / float(source_image->height())) * sy;
-	if (!isEqual(source_width_scale, 1.0) || !isEqual(source_height_scale, 1.0)) {
+	if (!isNear(source_width_scale, 1.0) || !isNear(source_height_scale, 1.0)) {
 		transform.scale(source_width_scale, source_height_scale);
 	}
 
