@@ -23,7 +23,7 @@ using namespace std;
 using namespace openshot;
 
 FrameMapper::FrameMapper(ReaderBase *reader, Fraction target, PulldownType target_pulldown, int target_sample_rate, int target_channels, ChannelLayout target_channel_layout) :
-		reader(reader), target(target), pulldown(target_pulldown), is_dirty(true), avr(NULL), parent_position(0.0), parent_start(0.0)
+		reader(reader), target(target), pulldown(target_pulldown), is_dirty(true), avr(NULL), parent_position(0.0), parent_start(0.0), previous_frame(0)
 {
 	// Set the original frame rate from the reader
 	original = Fraction(reader->info.fps.num, reader->info.fps.den);
@@ -41,6 +41,9 @@ FrameMapper::FrameMapper(ReaderBase *reader, Fraction target, PulldownType targe
 	info.width = reader->info.width;
 	info.height = reader->info.height;
 
+	// Enable/Disable audio (based on settings)
+	info.has_audio = info.sample_rate > 0 && info.channels > 0;
+
 	// Used to toggle odd / even fields
 	field_toggle = true;
 
@@ -60,17 +63,25 @@ FrameMapper::~FrameMapper() {
 /// Get the current reader
 ReaderBase* FrameMapper::Reader()
 {
-    if (reader)
-        return reader;
-    else
-        // Throw error if reader not initialized
-        throw ReaderClosed("No Reader has been initialized for FrameMapper.  Call Reader(*reader) before calling this method.");
+	if (reader)
+		return reader;
+	else
+		// Throw error if reader not initialized
+		throw ReaderClosed("No Reader has been initialized for FrameMapper.  Call Reader(*reader) before calling this method.");
 }
 
 void FrameMapper::AddField(int64_t frame)
 {
 	// Add a field, and toggle the odd / even field
-	AddField(Field(frame, field_toggle));
+    Field f = { frame, bool(field_toggle) };
+	AddField(f);
+}
+
+void FrameMapper::AddField(int64_t frame, bool isOdd)
+{
+	// Add a field, and toggle the odd / even field
+    Field f = { frame, isOdd };
+	AddField(f);
 }
 
 void FrameMapper::AddField(Field field)
@@ -84,14 +95,14 @@ void FrameMapper::AddField(Field field)
 
 // Clear both the fields & frames lists
 void FrameMapper::Clear() {
-    // Prevent async calls to the following code
-    const std::lock_guard<std::recursive_mutex> lock(getFrameMutex);
+	// Prevent async calls to the following code
+	const std::lock_guard<std::recursive_mutex> lock(getFrameMutex);
 
-    // Clear the fields & frames lists
-    fields.clear();
-    fields.shrink_to_fit();
-    frames.clear();
-    frames.shrink_to_fit();
+	// Clear the fields & frames lists
+	fields.clear();
+	fields.shrink_to_fit();
+	frames.clear();
+	frames.shrink_to_fit();
 }
 
 // Use the original and target frame rates and a pull-down technique to create
@@ -114,14 +125,14 @@ void FrameMapper::Init()
 	Clear();
 
 	// Find parent position (if any)
-    Clip *parent = (Clip *) ParentClip();
-    if (parent) {
-        parent_position = parent->Position();
-        parent_start = parent->Start();
-    } else {
-        parent_position = 0.0;
-        parent_start = 0.0;
-    }
+	Clip *parent = static_cast<Clip *>(ParentClip());
+	if (parent) {
+		parent_position = parent->Position();
+		parent_start = parent->Start();
+	} else {
+		parent_position = 0.0;
+		parent_start = 0.0;
+	}
 
 	// Mark as not dirty
 	is_dirty = false;
@@ -180,7 +191,7 @@ void FrameMapper::Init()
 
 					if (frame + 1 <= info.video_length)
 						// add field for next frame (if the next frame exists)
-						AddField(Field(frame + 1, field_toggle));
+						AddField(frame + 1, field_toggle);
 				}
 				else if (pulldown == PULLDOWN_NONE && field % frame_interval == 0)
 				{
@@ -241,8 +252,8 @@ void FrameMapper::Init()
 	}
 
 	// Loop through the target frames again (combining fields into frames)
-	Field Odd(0, true);		// temp field used to track the ODD field
-	Field Even(0, true);	// temp field used to track the EVEN field
+	Field Odd = {0, true};	// temp field used to track the ODD field
+	Field Even = {0, true};	// temp field used to track the EVEN field
 
 	// Variables used to remap audio samples
 	int64_t start_samples_frame = 1;
@@ -324,6 +335,13 @@ void FrameMapper::Init()
 	// Clear the internal fields list (no longer needed)
 	fields.clear();
 	fields.shrink_to_fit();
+
+	if (avr) {
+		// Delete resampler (if exists)
+		SWR_CLOSE(avr);
+		SWR_FREE(&avr);
+		avr = NULL;
+	}
 }
 
 MappedFrame FrameMapper::GetMappedFrame(int64_t TargetFrameNumber)
@@ -419,19 +437,24 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 	// Create a scoped lock, allowing only a single thread to run the following code at one time
 	const std::lock_guard<std::recursive_mutex> lock(getFrameMutex);
 
-    // Find parent properties (if any)
-    Clip *parent = (Clip *) ParentClip();
-    if (parent) {
-        float position = parent->Position();
-        float start = parent->Start();
-        if (parent_position != position || parent_start != start) {
-            // Force dirty if parent clip has moved or been trimmed
-            // since this heavily affects frame #s and audio mappings
-            is_dirty = true;
-        }
-    }
+	// Find parent properties (if any)
+	Clip *parent = static_cast<Clip *>(ParentClip());
+	bool is_increasing = true;
+	if (parent) {
+		float position = parent->Position();
+		float start = parent->Start();
+		if (parent_position != position || parent_start != start) {
+			// Force dirty if parent clip has moved or been trimmed
+			// since this heavily affects frame #s and audio mappings
+			is_dirty = true;
+		}
 
-    // Check if mappings are dirty (and need to be recalculated)
+		// Determine direction of parent clip at this frame (forward or reverse direction)
+		// This is important for reversing audio in our resampler, for smooth reversed audio.
+		is_increasing = parent->time.IsIncreasing(requested_frame);
+	}
+
+	// Check if mappings are dirty (and need to be recalculated)
 	if (is_dirty)
 		Init();
 
@@ -452,7 +475,6 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 	// Loop through all requested frames
 	for (int64_t frame_number = requested_frame; frame_number < requested_frame + minimum_frames; frame_number++)
 	{
-
 		// Debug output
 		ZmqLogger::Instance()->AppendDebugMethod(
 			"FrameMapper::GetFrame (inside omp for loop)",
@@ -462,10 +484,7 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 
 		// Get the mapped frame
 		MappedFrame mapped = GetMappedFrame(frame_number);
-		std::shared_ptr<Frame> mapped_frame;
-
-		// Get the mapped frame (keeping the sample rate and channels the same as the original... for the moment)
-		mapped_frame = GetOrCreateFrame(mapped.Odd.Frame);
+		std::shared_ptr<Frame> mapped_frame = GetOrCreateFrame(mapped.Odd.Frame);
 
 		// Get # of channels in the actual frame
 		int channels_in_frame = mapped_frame->GetAudioChannelsCount();
@@ -481,6 +500,7 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 			info.channels == mapped_frame->GetAudioChannelsCount() &&
 			info.channel_layout == mapped_frame->ChannelsLayout() &&
 			mapped.Samples.total == mapped_frame->GetAudioSamplesCount() &&
+			mapped.Samples.total == samples_in_frame && is_increasing &&
 			mapped.Samples.frame_start == mapped.Odd.Frame &&
 			mapped.Samples.sample_start == 0 &&
 			mapped_frame->number == frame_number &&// in some conditions (e.g. end of stream)
@@ -499,23 +519,24 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 
 
 		// Copy the image from the odd field
-		std::shared_ptr<Frame> odd_frame;
-		odd_frame = GetOrCreateFrame(mapped.Odd.Frame);
+		std::shared_ptr<Frame> odd_frame = mapped_frame;
 
-		if (odd_frame)
+		if (odd_frame && odd_frame->has_image_data)
 			frame->AddImage(std::make_shared<QImage>(*odd_frame->GetImage()), true);
 		if (mapped.Odd.Frame != mapped.Even.Frame) {
 			// Add even lines (if different than the previous image)
 			std::shared_ptr<Frame> even_frame;
 			even_frame = GetOrCreateFrame(mapped.Even.Frame);
-			if (even_frame)
-				frame->AddImage(
-					std::make_shared<QImage>(*even_frame->GetImage()), false);
+			if (even_frame && even_frame->has_image_data)
+				frame->AddImage(std::make_shared<QImage>(*even_frame->GetImage()), false);
 		}
 
+		// Determine if reader contains audio samples
+		bool reader_has_audio = frame->SampleRate() > 0 && frame->GetAudioChannelsCount() > 0;
+
 		// Resample audio on frame (if needed)
 		bool need_resampling = false;
-		if (info.has_audio &&
+		if ((info.has_audio && reader_has_audio) &&
 			(info.sample_rate != frame->SampleRate() ||
 			 info.channels != frame->GetAudioChannelsCount() ||
 			 info.channel_layout != frame->ChannelsLayout()))
@@ -527,40 +548,28 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 
 		if (need_resampling)
 		{
-			// Resampling needed, modify copy of SampleRange object that
-			// includes some additional input samples on first iteration,
-			// and continues the offset to ensure that the sample rate
-			// converter isn't input limited.
-			const int EXTRA_INPUT_SAMPLES = 100;
-
-			// Extend end sample count by an additional EXTRA_INPUT_SAMPLES samples
-			copy_samples.sample_end += EXTRA_INPUT_SAMPLES;
-			int samples_per_end_frame =
-				Frame::GetSamplesPerFrame(copy_samples.frame_end, original,
-				                          reader->info.sample_rate, reader->info.channels);
-			if (copy_samples.sample_end >= samples_per_end_frame)
-			{
-				// check for wrapping
-				copy_samples.frame_end++;
-				copy_samples.sample_end -= samples_per_end_frame;
-			}
-			copy_samples.total += EXTRA_INPUT_SAMPLES;
-
-			if (avr) {
-				// Sample rate conversion has been allocated on this clip, so
-				// this is not the first iteration. Extend start position by
-				// EXTRA_INPUT_SAMPLES to keep step with previous frame
-				copy_samples.sample_start += EXTRA_INPUT_SAMPLES;
-				int samples_per_start_frame =
-					Frame::GetSamplesPerFrame(copy_samples.frame_start, original,
-					                          reader->info.sample_rate, reader->info.channels);
-				if (copy_samples.sample_start >= samples_per_start_frame)
-				{
-					// check for wrapping
-					copy_samples.frame_start++;
-					copy_samples.sample_start -= samples_per_start_frame;
+			// Check for non-adjacent frame requests - so the resampler can be reset
+			if (abs(frame->number - previous_frame) > 1) {
+				if (avr) {
+					// Delete resampler (if exists)
+					SWR_CLOSE(avr);
+					SWR_FREE(&avr);
+					avr = NULL;
 				}
-				copy_samples.total -= EXTRA_INPUT_SAMPLES;
+			}
+
+			// Resampling needed, modify copy of SampleRange object that includes some additional input samples on
+			// first iteration, and continues the offset to ensure that the resampler is not input limited.
+			const int EXTRA_INPUT_SAMPLES = 64;
+
+			if (!avr) {
+				// This is the first iteration, and we need to extend # of samples for this frame
+				// Extend sample count range by an additional EXTRA_INPUT_SAMPLES
+				copy_samples.Extend(EXTRA_INPUT_SAMPLES, original, reader->info.sample_rate, reader->info.channels, is_increasing);
+			} else {
+				// Sample rate conversion has already been allocated on this clip, so
+				// this is not the first iteration. Shift position by EXTRA_INPUT_SAMPLES to correctly align samples
+				copy_samples.Shift(EXTRA_INPUT_SAMPLES, original, reader->info.sample_rate, reader->info.channels, is_increasing);
 			}
 		}
 
@@ -574,7 +583,11 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 			int number_to_copy = 0;
 
 			// number of original samples on this frame
-			std::shared_ptr<Frame> original_frame = GetOrCreateFrame(starting_frame);
+			std::shared_ptr<Frame> original_frame = mapped_frame;
+			if (starting_frame != original_frame->number) {
+				original_frame = GetOrCreateFrame(starting_frame);
+			}
+
 			int original_samples = original_frame->GetAudioSamplesCount();
 
 			// Loop through each channel
@@ -617,6 +630,10 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 			starting_frame++;
 		}
 
+		// Reverse audio (if needed)
+		if (!is_increasing)
+			frame->ReverseAudio();
+
 		// Resample audio on frame (if needed)
 		if (need_resampling)
 			// Resample audio and correct # of channels if needed
@@ -643,15 +660,15 @@ void FrameMapper::PrintMapping(std::ostream* out)
 	{
 		MappedFrame frame = frames[map - 1];
 		*out << "Target frame #: " << map
-		     << " mapped to original frame #:\t("
-		     << frame.Odd.Frame << " odd, "
-		     << frame.Even.Frame << " even)" << std::endl;
+			 << " mapped to original frame #:\t("
+			 << frame.Odd.Frame << " odd, "
+			 << frame.Even.Frame << " even)" << std::endl;
 
 		*out << "  - Audio samples mapped to frame "
-		     << frame.Samples.frame_start << ":"
-		     << frame.Samples.sample_start << " to frame "
-		     << frame.Samples.frame_end << ":"
-		     << frame.Samples.sample_end << endl;
+			 << frame.Samples.frame_start << ":"
+			 << frame.Samples.sample_start << " to frame "
+			 << frame.Samples.frame_end << ":"
+			 << frame.Samples.sample_end << endl;
 	}
 
 }
@@ -690,21 +707,21 @@ void FrameMapper::Close()
 		reader->Close();
 	}
 
-    // Clear the fields & frames lists
-    Clear();
+	// Clear the fields & frames lists
+	Clear();
 
-    // Mark as dirty
-    is_dirty = true;
+	// Mark as dirty
+	is_dirty = true;
 
-    // Clear cache
-    final_cache.Clear();
+	// Clear cache
+	final_cache.Clear();
 
-    // Deallocate resample buffer
-    if (avr) {
-        SWR_CLOSE(avr);
-        SWR_FREE(&avr);
-        avr = NULL;
-    }
+	// Deallocate resample buffer
+	if (avr) {
+		SWR_CLOSE(avr);
+		SWR_FREE(&avr);
+		avr = NULL;
+	}
 }
 
 
@@ -721,6 +738,9 @@ Json::Value FrameMapper::JsonValue() const {
 	// Create root json object
 	Json::Value root = ReaderBase::JsonValue(); // get parent properties
 	root["type"] = "FrameMapper";
+	if (reader) {
+		root["reader"] = reader->JsonValue();
+	}
 
 	// return JsonValue
 	return root;
@@ -735,6 +755,12 @@ void FrameMapper::SetJson(const std::string value) {
 		const Json::Value root = openshot::stringToJson(value);
 		// Set all values that match
 		SetJsonValue(root);
+
+		if (!root["reader"].isNull()) // does Json contain a reader?
+		{
+			// Placeholder to load reader
+			// TODO: need a createReader method for this and Clip JSON methods
+		}
 	}
 	catch (const std::exception& e)
 	{
@@ -785,6 +811,9 @@ void FrameMapper::ChangeMapping(Fraction target_fps, PulldownType target_pulldow
 	info.channels = target_channels;
 	info.channel_layout = target_channel_layout;
 
+	// Enable/Disable audio (based on settings)
+	info.has_audio = info.sample_rate > 0 && info.channels > 0;
+
 	// Clear cache
 	final_cache.Clear();
 
@@ -825,7 +854,7 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 	// Get audio sample array
 	float* frame_samples_float = NULL;
 	// Get samples interleaved together (c1 c2 c1 c2 c1 c2)
-	frame_samples_float = frame->GetInterleavedAudioSamples(sample_rate_in_frame, NULL, &samples_in_frame);
+	frame_samples_float = frame->GetInterleavedAudioSamples(&samples_in_frame);
 
 	// Calculate total samples
 	total_frame_samples = samples_in_frame * channels_in_frame;
@@ -851,21 +880,10 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 		frame_samples[s] = conv;
 	}
 
-
 	// Deallocate float array
 	delete[] frame_samples_float;
 	frame_samples_float = NULL;
 
-	ZmqLogger::Instance()->AppendDebugMethod(
-		"FrameMapper::ResampleMappedAudio (got sample data from frame)",
-		"frame->number", frame->number,
-		"total_frame_samples", total_frame_samples,
-		"target channels", info.channels,
-		"channels_in_frame", channels_in_frame,
-		"target sample_rate", info.sample_rate,
-		"samples_in_frame", samples_in_frame);
-
-
 	// Create input frame (and allocate arrays)
 	AVFrame *audio_frame = AV_ALLOCATE_FRAME();
 	AV_RESET_FRAME(audio_frame);
@@ -887,54 +905,36 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 	// Update total samples & input frame size (due to bigger or smaller data types)
 	total_frame_samples = Frame::GetSamplesPerFrame(AdjustFrameNumber(frame->number), target, info.sample_rate, info.channels);
 
-	ZmqLogger::Instance()->AppendDebugMethod(
-		"FrameMapper::ResampleMappedAudio (adjust # of samples)",
-		"total_frame_samples", total_frame_samples,
-		"info.sample_rate", info.sample_rate,
-		"sample_rate_in_frame", sample_rate_in_frame,
-		"info.channels", info.channels,
-		"channels_in_frame", channels_in_frame,
-		"original_frame_number", original_frame_number);
-
 	// Create output frame (and allocate arrays)
 	AVFrame *audio_converted = AV_ALLOCATE_FRAME();
 	AV_RESET_FRAME(audio_converted);
 	audio_converted->nb_samples = total_frame_samples;
 	av_samples_alloc(audio_converted->data, audio_converted->linesize, info.channels, total_frame_samples, AV_SAMPLE_FMT_S16, 0);
 
-	ZmqLogger::Instance()->AppendDebugMethod(
-		"FrameMapper::ResampleMappedAudio (preparing for resample)",
-		"in_sample_fmt", AV_SAMPLE_FMT_S16,
-		"out_sample_fmt", AV_SAMPLE_FMT_S16,
-		"in_sample_rate", sample_rate_in_frame,
-		"out_sample_rate", info.sample_rate,
-		"in_channels", channels_in_frame,
-		"out_channels", info.channels);
-
 	int nb_samples = 0;
 
-    // setup resample context
-    if (!avr) {
-        avr = SWR_ALLOC();
-        av_opt_set_int(avr, "in_channel_layout",  channel_layout_in_frame, 0);
-        av_opt_set_int(avr, "out_channel_layout", info.channel_layout,     0);
-        av_opt_set_int(avr, "in_sample_fmt",      AV_SAMPLE_FMT_S16,       0);
-        av_opt_set_int(avr, "out_sample_fmt",     AV_SAMPLE_FMT_S16,       0);
-        av_opt_set_int(avr, "in_sample_rate",     sample_rate_in_frame,    0);
-        av_opt_set_int(avr, "out_sample_rate",    info.sample_rate,        0);
-        av_opt_set_int(avr, "in_channels",        channels_in_frame,       0);
-        av_opt_set_int(avr, "out_channels",       info.channels,           0);
-        SWR_INIT(avr);
-    }
+	// setup resample context
+	if (!avr) {
+		avr = SWR_ALLOC();
+		av_opt_set_int(avr, "in_channel_layout",  channel_layout_in_frame, 0);
+		av_opt_set_int(avr, "out_channel_layout", info.channel_layout,	 0);
+		av_opt_set_int(avr, "in_sample_fmt",	  AV_SAMPLE_FMT_S16,	   0);
+		av_opt_set_int(avr, "out_sample_fmt",	 AV_SAMPLE_FMT_S16,	   0);
+		av_opt_set_int(avr, "in_sample_rate",	 sample_rate_in_frame,	0);
+		av_opt_set_int(avr, "out_sample_rate",	info.sample_rate,		0);
+		av_opt_set_int(avr, "in_channels",		channels_in_frame,	   0);
+		av_opt_set_int(avr, "out_channels",	   info.channels,		   0);
+		SWR_INIT(avr);
+	}
 
-    // Convert audio samples
-    nb_samples = SWR_CONVERT(avr,      // audio resample context
-        audio_converted->data,         // output data pointers
-        audio_converted->linesize[0],  // output plane size, in bytes. (0 if unknown)
-        audio_converted->nb_samples,   // maximum number of samples that the output buffer can hold
-        audio_frame->data,             // input data pointers
-        audio_frame->linesize[0],      // input plane size, in bytes (0 if unknown)
-        audio_frame->nb_samples);      // number of input samples to convert
+	// Convert audio samples
+	nb_samples = SWR_CONVERT(avr,	  // audio resample context
+		audio_converted->data,		 // output data pointers
+		audio_converted->linesize[0],  // output plane size, in bytes. (0 if unknown)
+		audio_converted->nb_samples,   // maximum number of samples that the output buffer can hold
+		audio_frame->data,			 // input data pointers
+		audio_frame->linesize[0],	  // input plane size, in bytes (0 if unknown)
+		audio_frame->nb_samples);	  // number of input samples to convert
 
 	// Create a new array (to hold all resampled S16 audio samples)
 	int16_t* resampled_samples = new int16_t[(nb_samples * info.channels)];
@@ -999,11 +999,6 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 
 		// Add samples to frame for this channel
 		frame->AddAudio(true, channel_filter, 0, channel_buffer, position, 1.0f);
-
-		ZmqLogger::Instance()->AppendDebugMethod(
-			"FrameMapper::ResampleMappedAudio (Add audio to channel)",
-			"number of samples", position,
-			"channel_filter", channel_filter);
 	}
 
 	// Update frame's audio meta data
@@ -1017,6 +1012,9 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 	// Delete arrays
 	delete[] resampled_samples;
 	resampled_samples = NULL;
+
+	// Keep track of last resampled frame
+	previous_frame = frame->number;
 }
 
 // Adjust frame number for Clip position and start (which can result in a different number)
@@ -1025,7 +1023,7 @@ int64_t FrameMapper::AdjustFrameNumber(int64_t clip_frame_number) {
 	// Get clip position from parent clip (if any)
 	float position = 0.0;
 	float start = 0.0;
-	Clip *parent = (Clip *) ParentClip();
+	Clip *parent = static_cast<Clip *>(ParentClip());
 	if (parent) {
 		position = parent->Position();
 		start = parent->Start();
